# -*- coding: utf-8 -*-

import copy
import json

from google.protobuf.json_format import MessageToJson


class Abi():
    """ Abi stores a contract abi."""
    def __init__(self, abi):
        self.__json = json.loads(MessageToJson(abi))
        self.__version = self.__json.get('version', None)
        self.__language = self.__json.get('language', None)
        self.__functions = self.__json.get('functions', [])
        self.__state_variables = self.__json.get('stateVariables', [])

    @property
    def version(self):
        return self.__version

    @property
    def language(self):
        return self.__language

    @property
    def functions(self):
        return self.__functions

    @property
    def state_variables(self):
        return self.__state_variables

    def json(self):
        return copy.deepcopy(self.__json)

    def __str__(self):
        return json.dumps(self.json(), indent=2)



# -*- coding: utf-8 -*-

import json
import socket

from google.protobuf.json_format import MessageToJson

from .block import Block
from ..status.peer_status import PeerStatus
from ..utils.encoding import encode_b58
from ..utils.converter import convert_ip_bytes_to_str


class Peer:
    def __init__(self):
        self.__info = None
        self.__address = None
        self.__port = None
        self.__id = None
        self.__state = None

    @property
    def info(self):
        return MessageToJson(self.__info)

    @info.setter
    def info(self, v):
        self.__info = v
        self.__address = convert_ip_bytes_to_str(v.address.address)
        self.__port = v.address.port
        self.__id = v.address.peerID
        self.__bestblock = Block(hash_value=v.bestblock.blockHash, height=v.bestblock.blockNo)
        self.__state = PeerStatus(v.state)

    @property
    def address(self):
        return self.__address

    @property
    def port(self):
        return self.__port

    @property
    def id(self):
        return self.__id

    @property
    def state(self):
        return self.__state

    @state.setter
    def state(self, s):
        self.__state = s

    def json(self):
        return {
            "Address": {
                "Address": str(self.__address),
                "Port": self.__port,
                "PeerID": encode_b58(self.__id),
            },
            "BestBlock": {
                "BlockNo": self.__bestblock.block_no,
                "BlockHash": str(self.__bestblock.hash),
            },
            "State": self.__state.name
        }

    def __str__(self):
        return json.dumps(self.json(), indent=2)



from .exception import AergoException


class ConversionException(AergoException):
    def __init__(self, msg):
        self.error_msg = msg
        self.exception_type = AergoException.Conv

    def __str__(self):
        print_err = "{0}: {1}".format(self.exception_type, self.error_msg)

        return print_err



import pytest

import ecdsa
import hashlib

from aergo.herapy.utils.encoding import decode_private_key
from aergo.herapy.utils.converter import convert_public_key_to_bytes
from aergo.herapy.account import Account
from aergo.herapy.obj.private_key import PrivateKey
from aergo.herapy.obj.address import Address


def test_import():
    # exported key from aergocli with password 1234
    # private key: 6i2n6TAtsKBaWSujYFzqkBCiP834j1u3nwmffZb8dxCxDcQaXAb
    # address: AmLnc5nhXjL3a3GUzv1Hb44LnGPvhZufDZ8s8oE9kazgbW6FgnUa
    exported_key = "47Nn73DSkBjHSfxvkJtiR3PACzQEZUN8qbjT41fYUfVEYipRZPwY4TnNRz5ahFHgN5naoJfJz"
    exported_key = decode_private_key(exported_key)

    acc = Account.decrypt_account(exported_key, '1234')
    assert str(acc.address) == "AmLnc5nhXjL3a3GUzv1Hb44LnGPvhZufDZ8s8oE9kazgbW6FgnUa"
    assert str(acc.private_key) == "6i2n6TAtsKBaWSujYFzqkBCiP834j1u3nwmffZb8dxCxDcQaXAb"

    assert acc.address == acc.private_key.address
    assert acc.public_key == acc.private_key.public_key
    assert acc.public_key.point == acc.private_key.public_key.point
    assert acc.public_key != acc.address.public_key
    assert acc.public_key.point == acc.address.public_key.point
    assert acc.private_key.public_key != acc.address.public_key
    assert acc.private_key.public_key.point == acc.address.public_key.point

    assert bytes(acc.address) == convert_public_key_to_bytes(acc.public_key)
    assert bytes(acc.address) == convert_public_key_to_bytes(acc.private_key.public_key)
    assert bytes(acc.address) == convert_public_key_to_bytes(acc.address.public_key)
    assert bytes(acc.private_key.address) == convert_public_key_to_bytes(acc.public_key)

    private_key = PrivateKey(pk="6i2n6TAtsKBaWSujYFzqkBCiP834j1u3nwmffZb8dxCxDcQaXAb")
    assert acc.public_key != private_key.public_key
    assert acc.public_key.point == private_key.public_key.point
    assert private_key.public_key != acc.address.public_key
    assert private_key.public_key.point == acc.address.public_key.point

    assert bytes(acc.address) == convert_public_key_to_bytes(private_key.public_key)
    assert bytes(private_key.address) == convert_public_key_to_bytes(acc.public_key)

    address = Address(None, empty=True)
    address.value = "AmLnc5nhXjL3a3GUzv1Hb44LnGPvhZufDZ8s8oE9kazgbW6FgnUa"
    assert acc.address != address
    assert bytes(acc.address) == bytes(address)
    assert private_key.address != address
    assert bytes(private_key.address) == bytes(address)

    assert acc.public_key != address.public_key
    assert acc.public_key.point == address.public_key.point
    assert private_key.public_key != address.public_key
    assert private_key.public_key.point == address.public_key.point

    assert bytes(acc.address) == convert_public_key_to_bytes(address.public_key)
    assert bytes(private_key.address) == convert_public_key_to_bytes(address.public_key)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: p2p.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import blockchain_pb2 as blockchain__pb2
from . import node_pb2 as node__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='p2p.proto',
  package='types',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\tp2p.proto\x12\x05types\x1a\x10\x62lockchain.proto\x1a\nnode.proto\"\xa8\x01\n\tMsgHeader\x12\x15\n\rclientVersion\x18\x01 \x01(\t\x12\x11\n\ttimestamp\x18\x02 \x01(\x03\x12\n\n\x02id\x18\x03 \x01(\t\x12\x0e\n\x06gossip\x18\x04 \x01(\x08\x12\x0e\n\x06peerID\x18\x05 \x01(\x0c\x12\x12\n\nnodePubKey\x18\x06 \x01(\x0c\x12\x0c\n\x04sign\x18\x07 \x01(\x0c\x12\x13\n\x0bsubprotocol\x18\x08 \x01(\r\x12\x0e\n\x06length\x18\t \x01(\r\"<\n\nP2PMessage\x12 \n\x06header\x18\x01 \x01(\x0b\x32\x10.types.MsgHeader\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"4\n\x04Ping\x12\x17\n\x0f\x62\x65st_block_hash\x18\x01 \x01(\x0c\x12\x13\n\x0b\x62\x65st_height\x18\x02 \x01(\x04\"1\n\x04Pong\x12\x15\n\rbestBlockHash\x18\x01 \x01(\x0c\x12\x12\n\nbestHeight\x18\x02 \x01(\x04\"\x9c\x01\n\x06Status\x12\"\n\x06sender\x18\x01 \x01(\x0b\x32\x12.types.PeerAddress\x12\x15\n\rbestBlockHash\x18\x02 \x01(\x0c\x12\x12\n\nbestHeight\x18\x03 \x01(\x04\x12\x0f\n\x07\x63hainID\x18\x04 \x01(\x0c\x12\x10\n\x08noExpose\x18\x05 \x01(\x08\x12\x0f\n\x07version\x18\x06 \x01(\t\x12\x0f\n\x07genesis\x18\x07 \x01(\x0c\"\x1f\n\x0cGoAwayNotice\x12\x0f\n\x07message\x18\x01 \x01(\t\"G\n\x10\x41\x64\x64ressesRequest\x12\"\n\x06sender\x18\x01 \x01(\x0b\x32\x12.types.PeerAddress\x12\x0f\n\x07maxSize\x18\x02 \x01(\r\"[\n\x11\x41\x64\x64ressesResponse\x12#\n\x06status\x18\x01 \x01(\x0e\x32\x13.types.ResultStatus\x12!\n\x05peers\x18\x02 \x03(\x0b\x32\x12.types.PeerAddress\"4\n\x0eNewBlockNotice\x12\x11\n\tblockHash\x18\x01 \x01(\x0c\x12\x0f\n\x07\x62lockNo\x18\x02 \x01(\x04\"W\n\x13\x42lockProducedNotice\x12\x12\n\nproducerID\x18\x01 \x01(\x0c\x12\x0f\n\x07\x62lockNo\x18\x02 \x01(\x04\x12\x1b\n\x05\x62lock\x18\x03 \x01(\x0b\x32\x0c.types.Block\"a\n\x16GetBlockHeadersRequest\x12\x0c\n\x04hash\x18\x01 \x01(\x0c\x12\x0e\n\x06height\x18\x02 \x01(\x04\x12\x0e\n\x06offset\x18\x03 \x01(\x04\x12\x0c\n\x04size\x18\x04 \x01(\r\x12\x0b\n\x03\x61sc\x18\x05 \x01(\x08\"\x84\x01\n\x17GetBlockHeadersResponse\x12#\n\x06status\x18\x01 \x01(\x0e\x32\x13.types.ResultStatus\x12\x0e\n\x06hashes\x18\x02 \x03(\x0c\x12#\n\x07headers\x18\x03 \x03(\x0b\x32\x12.types.BlockHeader\x12\x0f\n\x07hasNext\x18\x04 \x01(\x08\"!\n\x0fGetBlockRequest\x12\x0e\n\x06hashes\x18\x01 \x03(\x0c\"f\n\x10GetBlockResponse\x12#\n\x06status\x18\x01 \x01(\x0e\x32\x13.types.ResultStatus\x12\x1c\n\x06\x62locks\x18\x02 \x03(\x0b\x32\x0c.types.Block\x12\x0f\n\x07hasNext\x18\x03 \x01(\x08\")\n\x15NewTransactionsNotice\x12\x10\n\x08txHashes\x18\x01 \x03(\x0c\"(\n\x16GetTransactionsRequest\x12\x0e\n\x06hashes\x18\x01 \x03(\x0c\"w\n\x17GetTransactionsResponse\x12#\n\x06status\x18\x01 \x01(\x0e\x32\x13.types.ResultStatus\x12\x0e\n\x06hashes\x18\x02 \x03(\x0c\x12\x16\n\x03txs\x18\x03 \x03(\x0b\x32\t.types.Tx\x12\x0f\n\x07hasNext\x18\x04 \x01(\x08\"5\n\x11GetMissingRequest\x12\x0e\n\x06hashes\x18\x01 \x03(\x0c\x12\x10\n\x08stophash\x18\x02 \x01(\x0c\"$\n\x12GetAncestorRequest\x12\x0e\n\x06hashes\x18\x01 \x03(\x0c\"d\n\x13GetAncestorResponse\x12#\n\x06status\x18\x01 \x01(\x0e\x32\x13.types.ResultStatus\x12\x14\n\x0c\x61ncestorHash\x18\x02 \x01(\x0c\x12\x12\n\nancestorNo\x18\x03 \x01(\x04\"\x1e\n\x0bGetHashByNo\x12\x0f\n\x07\x62lockNo\x18\x01 \x01(\x04\"M\n\x13GetHashByNoResponse\x12#\n\x06status\x18\x01 \x01(\x0e\x32\x13.types.ResultStatus\x12\x11\n\tblockHash\x18\x02 \x01(\x0c\"F\n\x10GetHashesRequest\x12\x10\n\x08prevHash\x18\x01 \x01(\x0c\x12\x12\n\nprevNumber\x18\x02 \x01(\x04\x12\x0c\n\x04size\x18\x03 \x01(\x04\"Y\n\x11GetHashesResponse\x12#\n\x06status\x18\x01 \x01(\x0e\x32\x13.types.ResultStatus\x12\x0e\n\x06hashes\x18\x02 \x03(\x0c\x12\x0f\n\x07hasNext\x18\x03 \x01(\x08*\xbe\x02\n\x0cResultStatus\x12\x06\n\x02OK\x10\x00\x12\x0c\n\x08\x43\x41NCELED\x10\x01\x12\x0b\n\x07UNKNOWN\x10\x02\x12\x14\n\x10INVALID_ARGUMENT\x10\x03\x12\x15\n\x11\x44\x45\x41\x44LINE_EXCEEDED\x10\x04\x12\r\n\tNOT_FOUND\x10\x05\x12\x12\n\x0e\x41LREADY_EXISTS\x10\x06\x12\x15\n\x11PERMISSION_DENIED\x10\x07\x12\x16\n\x12RESOURCE_EXHAUSTED\x10\x08\x12\x17\n\x13\x46\x41ILED_PRECONDITION\x10\t\x12\x0b\n\x07\x41\x42ORTED\x10\n\x12\x10\n\x0cOUT_OF_RANGE\x10\x0b\x12\x11\n\rUNIMPLEMENTED\x10\x0c\x12\x0c\n\x08INTERNAL\x10\r\x12\x0f\n\x0bUNAVAILABLE\x10\x0e\x12\r\n\tDATA_LOSS\x10\x0f\x12\x13\n\x0fUNAUTHENTICATED\x10\x10\x62\x06proto3')
  ,
  dependencies=[blockchain__pb2.DESCRIPTOR,node__pb2.DESCRIPTOR,])

_RESULTSTATUS = _descriptor.EnumDescriptor(
  name='ResultStatus',
  full_name='types.ResultStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OK', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CANCELED', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INVALID_ARGUMENT', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEADLINE_EXCEEDED', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NOT_FOUND', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ALREADY_EXISTS', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PERMISSION_DENIED', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RESOURCE_EXHAUSTED', index=8, number=8,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILED_PRECONDITION', index=9, number=9,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ABORTED', index=10, number=10,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OUT_OF_RANGE', index=11, number=11,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNIMPLEMENTED', index=12, number=12,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INTERNAL', index=13, number=13,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNAVAILABLE', index=14, number=14,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DATA_LOSS', index=15, number=15,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNAUTHENTICATED', index=16, number=16,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1938,
  serialized_end=2256,
)
_sym_db.RegisterEnumDescriptor(_RESULTSTATUS)

ResultStatus = enum_type_wrapper.EnumTypeWrapper(_RESULTSTATUS)
OK = 0
CANCELED = 1
UNKNOWN = 2
INVALID_ARGUMENT = 3
DEADLINE_EXCEEDED = 4
NOT_FOUND = 5
ALREADY_EXISTS = 6
PERMISSION_DENIED = 7
RESOURCE_EXHAUSTED = 8
FAILED_PRECONDITION = 9
ABORTED = 10
OUT_OF_RANGE = 11
UNIMPLEMENTED = 12
INTERNAL = 13
UNAVAILABLE = 14
DATA_LOSS = 15
UNAUTHENTICATED = 16



_MSGHEADER = _descriptor.Descriptor(
  name='MsgHeader',
  full_name='types.MsgHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='clientVersion', full_name='types.MsgHeader.clientVersion', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='types.MsgHeader.timestamp', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='id', full_name='types.MsgHeader.id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gossip', full_name='types.MsgHeader.gossip', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='peerID', full_name='types.MsgHeader.peerID', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nodePubKey', full_name='types.MsgHeader.nodePubKey', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sign', full_name='types.MsgHeader.sign', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='subprotocol', full_name='types.MsgHeader.subprotocol', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='length', full_name='types.MsgHeader.length', index=8,
      number=9, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=51,
  serialized_end=219,
)


_P2PMESSAGE = _descriptor.Descriptor(
  name='P2PMessage',
  full_name='types.P2PMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='types.P2PMessage.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='types.P2PMessage.data', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=221,
  serialized_end=281,
)


_PING = _descriptor.Descriptor(
  name='Ping',
  full_name='types.Ping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='best_block_hash', full_name='types.Ping.best_block_hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='best_height', full_name='types.Ping.best_height', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=283,
  serialized_end=335,
)


_PONG = _descriptor.Descriptor(
  name='Pong',
  full_name='types.Pong',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bestBlockHash', full_name='types.Pong.bestBlockHash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bestHeight', full_name='types.Pong.bestHeight', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=337,
  serialized_end=386,
)


_STATUS = _descriptor.Descriptor(
  name='Status',
  full_name='types.Status',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sender', full_name='types.Status.sender', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bestBlockHash', full_name='types.Status.bestBlockHash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bestHeight', full_name='types.Status.bestHeight', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='chainID', full_name='types.Status.chainID', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noExpose', full_name='types.Status.noExpose', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='types.Status.version', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='genesis', full_name='types.Status.genesis', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=389,
  serialized_end=545,
)


_GOAWAYNOTICE = _descriptor.Descriptor(
  name='GoAwayNotice',
  full_name='types.GoAwayNotice',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='types.GoAwayNotice.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=547,
  serialized_end=578,
)


_ADDRESSESREQUEST = _descriptor.Descriptor(
  name='AddressesRequest',
  full_name='types.AddressesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sender', full_name='types.AddressesRequest.sender', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxSize', full_name='types.AddressesRequest.maxSize', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=580,
  serialized_end=651,
)


_ADDRESSESRESPONSE = _descriptor.Descriptor(
  name='AddressesResponse',
  full_name='types.AddressesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='types.AddressesResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='peers', full_name='types.AddressesResponse.peers', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=653,
  serialized_end=744,
)


_NEWBLOCKNOTICE = _descriptor.Descriptor(
  name='NewBlockNotice',
  full_name='types.NewBlockNotice',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='blockHash', full_name='types.NewBlockNotice.blockHash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blockNo', full_name='types.NewBlockNotice.blockNo', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=746,
  serialized_end=798,
)


_BLOCKPRODUCEDNOTICE = _descriptor.Descriptor(
  name='BlockProducedNotice',
  full_name='types.BlockProducedNotice',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='producerID', full_name='types.BlockProducedNotice.producerID', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blockNo', full_name='types.BlockProducedNotice.blockNo', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='block', full_name='types.BlockProducedNotice.block', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=800,
  serialized_end=887,
)


_GETBLOCKHEADERSREQUEST = _descriptor.Descriptor(
  name='GetBlockHeadersRequest',
  full_name='types.GetBlockHeadersRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='types.GetBlockHeadersRequest.hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='height', full_name='types.GetBlockHeadersRequest.height', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset', full_name='types.GetBlockHeadersRequest.offset', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='size', full_name='types.GetBlockHeadersRequest.size', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='asc', full_name='types.GetBlockHeadersRequest.asc', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=889,
  serialized_end=986,
)


_GETBLOCKHEADERSRESPONSE = _descriptor.Descriptor(
  name='GetBlockHeadersResponse',
  full_name='types.GetBlockHeadersResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='types.GetBlockHeadersResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hashes', full_name='types.GetBlockHeadersResponse.hashes', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='headers', full_name='types.GetBlockHeadersResponse.headers', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hasNext', full_name='types.GetBlockHeadersResponse.hasNext', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=989,
  serialized_end=1121,
)


_GETBLOCKREQUEST = _descriptor.Descriptor(
  name='GetBlockRequest',
  full_name='types.GetBlockRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hashes', full_name='types.GetBlockRequest.hashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1123,
  serialized_end=1156,
)


_GETBLOCKRESPONSE = _descriptor.Descriptor(
  name='GetBlockResponse',
  full_name='types.GetBlockResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='types.GetBlockResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blocks', full_name='types.GetBlockResponse.blocks', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hasNext', full_name='types.GetBlockResponse.hasNext', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1158,
  serialized_end=1260,
)


_NEWTRANSACTIONSNOTICE = _descriptor.Descriptor(
  name='NewTransactionsNotice',
  full_name='types.NewTransactionsNotice',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='txHashes', full_name='types.NewTransactionsNotice.txHashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1262,
  serialized_end=1303,
)


_GETTRANSACTIONSREQUEST = _descriptor.Descriptor(
  name='GetTransactionsRequest',
  full_name='types.GetTransactionsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hashes', full_name='types.GetTransactionsRequest.hashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1305,
  serialized_end=1345,
)


_GETTRANSACTIONSRESPONSE = _descriptor.Descriptor(
  name='GetTransactionsResponse',
  full_name='types.GetTransactionsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='types.GetTransactionsResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hashes', full_name='types.GetTransactionsResponse.hashes', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='txs', full_name='types.GetTransactionsResponse.txs', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hasNext', full_name='types.GetTransactionsResponse.hasNext', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1347,
  serialized_end=1466,
)


_GETMISSINGREQUEST = _descriptor.Descriptor(
  name='GetMissingRequest',
  full_name='types.GetMissingRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hashes', full_name='types.GetMissingRequest.hashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stophash', full_name='types.GetMissingRequest.stophash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1468,
  serialized_end=1521,
)


_GETANCESTORREQUEST = _descriptor.Descriptor(
  name='GetAncestorRequest',
  full_name='types.GetAncestorRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hashes', full_name='types.GetAncestorRequest.hashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1523,
  serialized_end=1559,
)


_GETANCESTORRESPONSE = _descriptor.Descriptor(
  name='GetAncestorResponse',
  full_name='types.GetAncestorResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='types.GetAncestorResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ancestorHash', full_name='types.GetAncestorResponse.ancestorHash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ancestorNo', full_name='types.GetAncestorResponse.ancestorNo', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1561,
  serialized_end=1661,
)


_GETHASHBYNO = _descriptor.Descriptor(
  name='GetHashByNo',
  full_name='types.GetHashByNo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='blockNo', full_name='types.GetHashByNo.blockNo', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1663,
  serialized_end=1693,
)


_GETHASHBYNORESPONSE = _descriptor.Descriptor(
  name='GetHashByNoResponse',
  full_name='types.GetHashByNoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='types.GetHashByNoResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blockHash', full_name='types.GetHashByNoResponse.blockHash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1695,
  serialized_end=1772,
)


_GETHASHESREQUEST = _descriptor.Descriptor(
  name='GetHashesRequest',
  full_name='types.GetHashesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='prevHash', full_name='types.GetHashesRequest.prevHash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prevNumber', full_name='types.GetHashesRequest.prevNumber', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='size', full_name='types.GetHashesRequest.size', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1774,
  serialized_end=1844,
)


_GETHASHESRESPONSE = _descriptor.Descriptor(
  name='GetHashesResponse',
  full_name='types.GetHashesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='types.GetHashesResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hashes', full_name='types.GetHashesResponse.hashes', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hasNext', full_name='types.GetHashesResponse.hasNext', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1846,
  serialized_end=1935,
)

_P2PMESSAGE.fields_by_name['header'].message_type = _MSGHEADER
_STATUS.fields_by_name['sender'].message_type = node__pb2._PEERADDRESS
_ADDRESSESREQUEST.fields_by_name['sender'].message_type = node__pb2._PEERADDRESS
_ADDRESSESRESPONSE.fields_by_name['status'].enum_type = _RESULTSTATUS
_ADDRESSESRESPONSE.fields_by_name['peers'].message_type = node__pb2._PEERADDRESS
_BLOCKPRODUCEDNOTICE.fields_by_name['block'].message_type = blockchain__pb2._BLOCK
_GETBLOCKHEADERSRESPONSE.fields_by_name['status'].enum_type = _RESULTSTATUS
_GETBLOCKHEADERSRESPONSE.fields_by_name['headers'].message_type = blockchain__pb2._BLOCKHEADER
_GETBLOCKRESPONSE.fields_by_name['status'].enum_type = _RESULTSTATUS
_GETBLOCKRESPONSE.fields_by_name['blocks'].message_type = blockchain__pb2._BLOCK
_GETTRANSACTIONSRESPONSE.fields_by_name['status'].enum_type = _RESULTSTATUS
_GETTRANSACTIONSRESPONSE.fields_by_name['txs'].message_type = blockchain__pb2._TX
_GETANCESTORRESPONSE.fields_by_name['status'].enum_type = _RESULTSTATUS
_GETHASHBYNORESPONSE.fields_by_name['status'].enum_type = _RESULTSTATUS
_GETHASHESRESPONSE.fields_by_name['status'].enum_type = _RESULTSTATUS
DESCRIPTOR.message_types_by_name['MsgHeader'] = _MSGHEADER
DESCRIPTOR.message_types_by_name['P2PMessage'] = _P2PMESSAGE
DESCRIPTOR.message_types_by_name['Ping'] = _PING
DESCRIPTOR.message_types_by_name['Pong'] = _PONG
DESCRIPTOR.message_types_by_name['Status'] = _STATUS
DESCRIPTOR.message_types_by_name['GoAwayNotice'] = _GOAWAYNOTICE
DESCRIPTOR.message_types_by_name['AddressesRequest'] = _ADDRESSESREQUEST
DESCRIPTOR.message_types_by_name['AddressesResponse'] = _ADDRESSESRESPONSE
DESCRIPTOR.message_types_by_name['NewBlockNotice'] = _NEWBLOCKNOTICE
DESCRIPTOR.message_types_by_name['BlockProducedNotice'] = _BLOCKPRODUCEDNOTICE
DESCRIPTOR.message_types_by_name['GetBlockHeadersRequest'] = _GETBLOCKHEADERSREQUEST
DESCRIPTOR.message_types_by_name['GetBlockHeadersResponse'] = _GETBLOCKHEADERSRESPONSE
DESCRIPTOR.message_types_by_name['GetBlockRequest'] = _GETBLOCKREQUEST
DESCRIPTOR.message_types_by_name['GetBlockResponse'] = _GETBLOCKRESPONSE
DESCRIPTOR.message_types_by_name['NewTransactionsNotice'] = _NEWTRANSACTIONSNOTICE
DESCRIPTOR.message_types_by_name['GetTransactionsRequest'] = _GETTRANSACTIONSREQUEST
DESCRIPTOR.message_types_by_name['GetTransactionsResponse'] = _GETTRANSACTIONSRESPONSE
DESCRIPTOR.message_types_by_name['GetMissingRequest'] = _GETMISSINGREQUEST
DESCRIPTOR.message_types_by_name['GetAncestorRequest'] = _GETANCESTORREQUEST
DESCRIPTOR.message_types_by_name['GetAncestorResponse'] = _GETANCESTORRESPONSE
DESCRIPTOR.message_types_by_name['GetHashByNo'] = _GETHASHBYNO
DESCRIPTOR.message_types_by_name['GetHashByNoResponse'] = _GETHASHBYNORESPONSE
DESCRIPTOR.message_types_by_name['GetHashesRequest'] = _GETHASHESREQUEST
DESCRIPTOR.message_types_by_name['GetHashesResponse'] = _GETHASHESRESPONSE
DESCRIPTOR.enum_types_by_name['ResultStatus'] = _RESULTSTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MsgHeader = _reflection.GeneratedProtocolMessageType('MsgHeader', (_message.Message,), {
  'DESCRIPTOR' : _MSGHEADER,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.MsgHeader)
  })
_sym_db.RegisterMessage(MsgHeader)

P2PMessage = _reflection.GeneratedProtocolMessageType('P2PMessage', (_message.Message,), {
  'DESCRIPTOR' : _P2PMESSAGE,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.P2PMessage)
  })
_sym_db.RegisterMessage(P2PMessage)

Ping = _reflection.GeneratedProtocolMessageType('Ping', (_message.Message,), {
  'DESCRIPTOR' : _PING,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.Ping)
  })
_sym_db.RegisterMessage(Ping)

Pong = _reflection.GeneratedProtocolMessageType('Pong', (_message.Message,), {
  'DESCRIPTOR' : _PONG,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.Pong)
  })
_sym_db.RegisterMessage(Pong)

Status = _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), {
  'DESCRIPTOR' : _STATUS,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.Status)
  })
_sym_db.RegisterMessage(Status)

GoAwayNotice = _reflection.GeneratedProtocolMessageType('GoAwayNotice', (_message.Message,), {
  'DESCRIPTOR' : _GOAWAYNOTICE,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GoAwayNotice)
  })
_sym_db.RegisterMessage(GoAwayNotice)

AddressesRequest = _reflection.GeneratedProtocolMessageType('AddressesRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDRESSESREQUEST,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.AddressesRequest)
  })
_sym_db.RegisterMessage(AddressesRequest)

AddressesResponse = _reflection.GeneratedProtocolMessageType('AddressesResponse', (_message.Message,), {
  'DESCRIPTOR' : _ADDRESSESRESPONSE,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.AddressesResponse)
  })
_sym_db.RegisterMessage(AddressesResponse)

NewBlockNotice = _reflection.GeneratedProtocolMessageType('NewBlockNotice', (_message.Message,), {
  'DESCRIPTOR' : _NEWBLOCKNOTICE,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.NewBlockNotice)
  })
_sym_db.RegisterMessage(NewBlockNotice)

BlockProducedNotice = _reflection.GeneratedProtocolMessageType('BlockProducedNotice', (_message.Message,), {
  'DESCRIPTOR' : _BLOCKPRODUCEDNOTICE,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.BlockProducedNotice)
  })
_sym_db.RegisterMessage(BlockProducedNotice)

GetBlockHeadersRequest = _reflection.GeneratedProtocolMessageType('GetBlockHeadersRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETBLOCKHEADERSREQUEST,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetBlockHeadersRequest)
  })
_sym_db.RegisterMessage(GetBlockHeadersRequest)

GetBlockHeadersResponse = _reflection.GeneratedProtocolMessageType('GetBlockHeadersResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETBLOCKHEADERSRESPONSE,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetBlockHeadersResponse)
  })
_sym_db.RegisterMessage(GetBlockHeadersResponse)

GetBlockRequest = _reflection.GeneratedProtocolMessageType('GetBlockRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETBLOCKREQUEST,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetBlockRequest)
  })
_sym_db.RegisterMessage(GetBlockRequest)

GetBlockResponse = _reflection.GeneratedProtocolMessageType('GetBlockResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETBLOCKRESPONSE,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetBlockResponse)
  })
_sym_db.RegisterMessage(GetBlockResponse)

NewTransactionsNotice = _reflection.GeneratedProtocolMessageType('NewTransactionsNotice', (_message.Message,), {
  'DESCRIPTOR' : _NEWTRANSACTIONSNOTICE,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.NewTransactionsNotice)
  })
_sym_db.RegisterMessage(NewTransactionsNotice)

GetTransactionsRequest = _reflection.GeneratedProtocolMessageType('GetTransactionsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONSREQUEST,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetTransactionsRequest)
  })
_sym_db.RegisterMessage(GetTransactionsRequest)

GetTransactionsResponse = _reflection.GeneratedProtocolMessageType('GetTransactionsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTRANSACTIONSRESPONSE,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetTransactionsResponse)
  })
_sym_db.RegisterMessage(GetTransactionsResponse)

GetMissingRequest = _reflection.GeneratedProtocolMessageType('GetMissingRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETMISSINGREQUEST,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetMissingRequest)
  })
_sym_db.RegisterMessage(GetMissingRequest)

GetAncestorRequest = _reflection.GeneratedProtocolMessageType('GetAncestorRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETANCESTORREQUEST,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetAncestorRequest)
  })
_sym_db.RegisterMessage(GetAncestorRequest)

GetAncestorResponse = _reflection.GeneratedProtocolMessageType('GetAncestorResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETANCESTORRESPONSE,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetAncestorResponse)
  })
_sym_db.RegisterMessage(GetAncestorResponse)

GetHashByNo = _reflection.GeneratedProtocolMessageType('GetHashByNo', (_message.Message,), {
  'DESCRIPTOR' : _GETHASHBYNO,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetHashByNo)
  })
_sym_db.RegisterMessage(GetHashByNo)

GetHashByNoResponse = _reflection.GeneratedProtocolMessageType('GetHashByNoResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETHASHBYNORESPONSE,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetHashByNoResponse)
  })
_sym_db.RegisterMessage(GetHashByNoResponse)

GetHashesRequest = _reflection.GeneratedProtocolMessageType('GetHashesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETHASHESREQUEST,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetHashesRequest)
  })
_sym_db.RegisterMessage(GetHashesRequest)

GetHashesResponse = _reflection.GeneratedProtocolMessageType('GetHashesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETHASHESRESPONSE,
  '__module__' : 'p2p_pb2'
  # @@protoc_insertion_point(class_scope:types.GetHashesResponse)
  })
_sym_db.RegisterMessage(GetHashesResponse)


# @@protoc_insertion_point(module_scope)



import sys
import traceback

import aergo.herapy as herapy


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)
    print(*args, **kwargs)


def run():
    try:
        aergo = herapy.Aergo()

        print("------ Export New Account -----------")
        aergo.new_account()
        new_exp_txt = aergo.export_account(password="1234")
        print("Exported txt is {}".format(new_exp_txt))

        print("------ Import Account -----------")
        try:
            aergo.import_account(new_exp_txt, password='test')
        except herapy.errors.GeneralException:
            print("It should be failed.")

        print("------ Import Account -----------")
        try:
            account = aergo.import_account(new_exp_txt, password='1234')
            print("Account private key is {}".format(account.private_key))
            print("Account address is {}".format(account.address))
        except herapy.errors.GeneralException:
            print("It should be failed.")

        print("------ Import Account with Exported Data -----------")
        #exported_txt = "485ccQXjmT3JeUHV16n16LzAJhhfHHkv9HU9k7c5PeJDyPMAdLcCu8Yqws19UzMP4K4Rq2MkQ"
        exported_txt = "MNxKz7jPTaWW8xZc6HSbgTPXDj6yGq6PrLbboSyV5psBfKyRmbo5qoVHgWTEXucisaUy8Y3PGf4UBc"
        print("Exported Data is {}".format(exported_txt))
        account = aergo.import_account(exported_txt, password='1234')
        print("Account private key is {}".format(account.private_key))
        print("Account address is {}".format(account.address))

        print("------ Export Account -----------")
        new_exp_txt = aergo.export_account(password='1234')
        print("Exported txt is {}".format(new_exp_txt))

        print("------ Connect AERGO -----------")
        aergo.connect('localhost:7845')

        print("------ Get Account State -----------")
        a = aergo.get_account(address=account.address)
        print("  > account state of Import account")
        print("    - balance        = {}".format(a.balance))
        print("    - nonce          = {}".format(a.nonce))
        print("    - code hash      = {}".format(a.code_hash))
        print("    - storage root   = {}".format(a.storage_root))

        print("------ Disconnect AERGO -----------")
        aergo.disconnect()
    except Exception as e:
        eprint(e)
        traceback.print_exception(*sys.exc_info())


if __name__ == '__main__':
    run()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: account.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='account.proto',
  package='types',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\raccount.proto\x12\x05types\"\x1a\n\x07\x41\x63\x63ount\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0c\"/\n\x0b\x41\x63\x63ountList\x12 \n\x08\x61\x63\x63ounts\x18\x01 \x03(\x0b\x32\x0e.types.Accountb\x06proto3')
)




_ACCOUNT = _descriptor.Descriptor(
  name='Account',
  full_name='types.Account',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='types.Account.address', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24,
  serialized_end=50,
)


_ACCOUNTLIST = _descriptor.Descriptor(
  name='AccountList',
  full_name='types.AccountList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accounts', full_name='types.AccountList.accounts', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=52,
  serialized_end=99,
)

_ACCOUNTLIST.fields_by_name['accounts'].message_type = _ACCOUNT
DESCRIPTOR.message_types_by_name['Account'] = _ACCOUNT
DESCRIPTOR.message_types_by_name['AccountList'] = _ACCOUNTLIST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Account = _reflection.GeneratedProtocolMessageType('Account', (_message.Message,), {
  'DESCRIPTOR' : _ACCOUNT,
  '__module__' : 'account_pb2'
  # @@protoc_insertion_point(class_scope:types.Account)
  })
_sym_db.RegisterMessage(Account)

AccountList = _reflection.GeneratedProtocolMessageType('AccountList', (_message.Message,), {
  'DESCRIPTOR' : _ACCOUNTLIST,
  '__module__' : 'account_pb2'
  # @@protoc_insertion_point(class_scope:types.AccountList)
  })
_sym_db.RegisterMessage(AccountList)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-

import copy
import json

from google.protobuf.json_format import MessageToJson


class EnterpriseConfig:
    """ EnterpriseConfig stores the result of GetEnterpriseConfig. """
    def __init__(self, config):
        self.__json = json.loads(MessageToJson(config))
        self.__key = self.__json.get('key', None)
        self.__on = self.__json.get('on', False)
        self.__values = self.__json.get("values", [])

    def __str__(self):
        return json.dumps(self.json(), indent=2)

    def json(self):
        return copy.deepcopy(self.__json)

    @property
    def key(self):
        return self.__key

    @property
    def on(self):
        return self.__on

    @property
    def values(self):
        return self.__values



# -*- coding: utf-8 -*-

from .stream import Stream
from .block import Block


class BlockMetaStream(Stream):
    def __init__(self, block_meta_stream):
        super().__init__(block_meta_stream)

    def __next__(self):
        grpc_block_meta = next(self._grpc_stream)
        return Block(hash_value=grpc_block_meta.hash,
                     grpc_block_header=grpc_block_meta.header,
                     tx_cnt=grpc_block_meta.txcount)




import pytest

import ecdsa
import hashlib

from aergo.herapy.obj.address import Address
from aergo.herapy.utils.encoding import encode_address, decode_address
from aergo.herapy.utils.converter import convert_public_key_to_bytes


def test_error():
    with pytest.raises(AssertionError):
        Address(pubkey=None)

    with pytest.raises(TypeError):
        Address(pubkey=1234)

    with pytest.raises(ValueError):
        Address(pubkey="1234")

    with pytest.raises(TypeError):
        Address(pubkey=["1234"])


def test_success():
    sk = ecdsa.SigningKey.generate(curve=ecdsa.SECP256k1,
                                   hashfunc=hashlib.sha256)
    addr = Address(pubkey=sk.privkey.public_key)
    assert type(addr) is Address
    assert isinstance(str(addr), str)
    assert isinstance(bytes(addr), bytes)
    assert bytes(addr) == addr.value
    assert bytes(addr) == convert_public_key_to_bytes(pubkey=addr.public_key,
                                                      curve=addr.curve,
                                                      compressed=True)
    with pytest.raises(ValueError):
        addr.value = str(addr)

    addr_str = str(addr)

    addr2 = Address(None, empty=True)
    addr2.value = addr_str
    assert type(addr2) is Address
    assert isinstance(str(addr2), str)
    assert isinstance(bytes(addr2), bytes)
    assert bytes(addr2) == addr2.value
    assert bytes(addr2) == convert_public_key_to_bytes(pubkey=addr2.public_key,
                                                       curve=addr2.curve,
                                                       compressed=True)

    assert addr2 != addr
    assert addr2.value == addr.value
    assert str(addr2) == str(addr)
    assert bytes(addr2) == bytes(addr)
    assert addr2.public_key.point == addr.public_key.point
    assert convert_public_key_to_bytes(pubkey=addr.public_key,
                                       curve=addr.curve,
                                       compressed=False) == \
           convert_public_key_to_bytes(pubkey=addr2.public_key,
                                       curve=addr2.curve,
                                       compressed=False)

    pubkey3 = convert_public_key_to_bytes(pubkey=addr.public_key,
                                          curve=addr.curve,
                                          compressed=False)
    pubkey4 = convert_public_key_to_bytes(pubkey=addr2.public_key,
                                          curve=addr.curve,
                                          compressed=True)
    addr3 = Address(pubkey=pubkey3)
    assert addr3.value == addr.value
    assert bytes(addr3) == bytes(addr)
    addr4 = Address(pubkey=pubkey4)
    assert addr4.value == addr2.value
    assert bytes(addr4) == bytes(addr2)

    assert addr4 != addr3
    assert str(addr4) == str(addr3)
    assert bytes(addr4) == bytes(addr3)
    assert addr4.public_key.point == addr3.public_key.point

    addr5 = Address(None, empty=True)
    addr5.value = bytes(addr3)
    assert addr4 != addr5
    assert str(addr4) == str(addr5)
    assert bytes(addr4) == bytes(addr5)
    assert addr4.public_key.point == addr5.public_key.point


def test_encode_empty():
    addr = Address(None, empty=True)
    assert str(addr) == ''

    addr = Address(None, empty=True)
    addr.value = bytes([])
    assert str(addr) == ''


def test_govname():
    addr = Address(None, empty=True)
    name = 'abcdetest'
    addr.value = name.encode()
    assert len(str(addr)) == len(name)

    addr = Address(None, empty=True)
    baddr = bytes([0x61, 0x65, 0x72, 0x67, 0x6f, 0x2e, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x70, 0x72, 0x69, 0x73, 0x65])
    addr.value = baddr
    assert str(addr) == 'aergo.enterprise'
    assert str(addr) == Address.encode(baddr)
    assert bytes(addr) == Address.decode('aergo.enterprise')
    assert baddr == Address.decode('aergo.enterprise')

    addr = Address(None, empty=True)
    addr.value = 'aergo.system'
    assert str(addr) == 'aergo.system'
    assert bytes(addr) == Address.decode('aergo.system')



import sys
import traceback
import time

import aergo.herapy as herapy


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)
    print(*args, **kwargs)


def run():
    try:
        aergo = herapy.Aergo()

        print("------ Connect AERGO -----------")
        aergo.connect('localhost:7845')

        exp_acc = "47eY3XwQ2UgjNfWCXhhySwmZNvrFf3MASEE6hdqxdG5ioQPWKsVNdZyecFqbCVt752ANPAUpY"
        password = "1234"

        aergo.import_account(exported_data=exp_acc, password=password)
        print(str(aergo.account.address))

        rs = aergo.get_name_info(name="te0123456789")
        print(rs.name)
        print(rs)

        print("------ Disconnect AERGO -----------")
        aergo.disconnect()
    except Exception as e:
        eprint(e)
        traceback.print_exception(*sys.exc_info())


if __name__ == '__main__':
    run()



# -*- coding: utf-8 -*-

"""Enumeration of Commit Status."""

import enum

from ..grpc import rpc_pb2


@enum.unique
class CommitStatus(enum.IntEnum):
    """
    TX_OK = 0
    TX_NONCE_TOO_LOW = 1
    TX_ALREADY_EXISTS = 2
    TX_INVALID_HASH = 3
    TX_INVALID_SIGN = 4
    TX_INVALID_FORMAT = 5
    TX_INSUFFICIENT_BALANCE = 6
    TX_HAS_SAME_NONCE = 7
    TX_INTERNAL_ERROR = 9
    """
    TX_OK = rpc_pb2.TX_OK
    TX_NONCE_TOO_LOW = rpc_pb2.TX_NONCE_TOO_LOW
    TX_ALREADY_EXISTS = rpc_pb2.TX_ALREADY_EXISTS
    TX_INVALID_HASH = rpc_pb2.TX_INVALID_HASH
    TX_INVALID_SIGN = rpc_pb2.TX_INVALID_SIGN
    TX_INVALID_FORMAT = rpc_pb2.TX_INVALID_FORMAT
    TX_INSUFFICIENT_BALANCE = rpc_pb2.TX_INSUFFICIENT_BALANCE
    TX_HAS_SAME_NONCE = rpc_pb2.TX_HAS_SAME_NONCE
    TX_INTERNAL_ERROR = rpc_pb2.TX_INTERNAL_ERROR




import sys
import traceback

import aergo.herapy as herapy
import hashlib


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)
    print(*args, **kwargs)


def run():
    try:
        aergo = herapy.Aergo()

        aergo.new_account(skip_state=True)
        private_key_bytes = bytes(aergo.account.private_key)
        print("int(private key bytes) = {}".format(herapy.utils.bytes_to_int_str(private_key_bytes)))

        aergo.import_account(exported_data="MNxKz7jkvDGd61cifsmq2M21XL1uyvEcFS89BQpDMx57n44K3pfySoumSTdw7MS6HgfNy3ToJ7ravf", password="1234")
        private_key_bytes = bytes(aergo.account.private_key)
        signing_key = aergo.account.private_key.get_signing_key()
        verifying_key =  signing_key.get_verifying_key()
        print("to pem = {}".format(verifying_key.to_pem()))
        print("int(private key bytes) = {}".format(herapy.utils.bytes_to_int_str(private_key_bytes)))
        print("Address = {}".format(str(aergo.account.address)))

        pubkey_bytes_head = "\x08\x02\x12".encode("latin-1")
        print("int(public key bytes head) = {}".format(herapy.utils.bytes_to_int_str(pubkey_bytes_head)))

        print("Compress")
        pubkey_bytes = herapy.utils.convert_public_key_to_bytes(aergo.account.public_key)
        print("bytes(public key) = {}".format(pubkey_bytes))
        print("int(public key bytes) = {}".format(herapy.utils.bytes_to_int_str(pubkey_bytes)))

        p2p_pubkey_bytes = pubkey_bytes_head + len(pubkey_bytes).to_bytes(length=1, byteorder='big') + pubkey_bytes
        print("bytes(p2p public key) = {}".format(p2p_pubkey_bytes))
        print("int(p2p public key bytes) = {}".format(herapy.utils.bytes_to_int_str(p2p_pubkey_bytes)))
        pubkey_txt = herapy.utils.encode_b64(p2p_pubkey_bytes)
        print("p2p public key = {}".format(pubkey_txt))

        '''
        print("No compress")
        pubkey_bytes = herapy.utils.convert_public_key_to_bytes(aergo.account.public_key, compressed=False)
        print("bytes(public key) = {}".format(pubkey_bytes))
        pubkey_txt = herapy.utils.encode_b64(pubkey_bytes)
        print("public key = {}".format(pubkey_txt))
        '''

        print("generate ID")
        '''
        hash = hashlib.sha256()
        hash.update(p2p_pubkey_bytes)
        id_bytes = hash.digest()
        '''
        id_bytes = "\x00\x25".encode("latin-1") + p2p_pubkey_bytes
        print("int(id bytes) = {}".format(herapy.utils.bytes_to_int_str(id_bytes)))
        id = herapy.utils.encode_b58(id_bytes)
        print("id = {}".format(id))

        aergo.new_account(skip_state=True)
        pubkey_bytes = herapy.utils.convert_public_key_to_bytes(aergo.account.public_key)
        print("bytes(public key) = {}".format(pubkey_bytes))


        print("------ Connect AERGO -----------")
        aergo.connect('localhost:7845')

        print("------ Get Accounts without State in Node -----------")
        accounts = aergo.get_node_accounts(skip_state=True)
        for i, account in enumerate(accounts):
            print("  > account state : {}".format(account))
            print("    - balance")
            print("        + aergo              = {}".format(account.balance.aergo))
            print("        + gaer               = {}".format(account.balance.gaer))
            print("        + aer                = {}".format(account.balance.aer))
            print("    - nonce                  = {}".format(account.nonce))
            print("    - code hash              = {}".format(account.code_hash))
            print("    - storage root           = {}".format(account.storage_root))
            print("    - sql recovery point     = {}".format(account.sql_recovery_point))

        print("------ Get Accounts in Node -----------")
        accounts = aergo.get_node_accounts()
        for i, account in enumerate(accounts):
            print("  > account state : {}".format(account))
            print("    - balance")
            print("        + aergo              = {}".format(account.balance.aergo))
            print("        + gaer               = {}".format(account.balance.gaer))
            print("        + aer                = {}".format(account.balance.aer))
            print("    - nonce                  = {}".format(account.nonce))
            print("    - code hash              = {}".format(account.code_hash))
            print("    - storage root           = {}".format(account.storage_root))
            print("    - sql recovery point     = {}".format(account.sql_recovery_point))

        print("------ Create Account -----------")
        account = aergo.new_account()

        print("Private Key      = {}".format(account.private_key))
        print("Public Key       = {}".format(account.public_key))
        print("Address          = {}".format(account.address))

        print("------ Get Account State -----------")
        aergo.get_account()
        print("  > account state in 'aergo'")
        print("    - balance")
        print("        + aergo              = {}".format(account.balance.aergo))
        print("        + gaer               = {}".format(account.balance.gaer))
        print("        + aer                = {}".format(account.balance.aer))
        print("    - nonce                  = {}".format(account.nonce))
        print("    - code hash              = {}".format(account.code_hash))
        print("    - storage root           = {}".format(account.storage_root))
        print("    - sql recovery point     = {}".format(account.sql_recovery_point))
        print(account)

        print("------ Get Configured Account -----------")
        accounts = [
            {
                "private_key": "eHoEcHnaxpGpgzknXjuwon8VFVrLkKHC4FckGuGkQ8depiDDfyUAWC3L",
                "address": "AmPZKCJpT98V9Tc8dBUbRg78M1jgoB1ZEh97Rs1r5KewPcCiURf7",
            },
        ]
        for i, account in enumerate(accounts):
            print("  [{}]".format(i))
            print("    > private key   : {}".format(account['private_key']))
            print("    > address       : {}".format(account['address']))

            # check account state
            a = aergo.get_account(address=account['address'])
            print("    > account state : {}".format(a))
            print("    - balance")
            print("        + aergo              = {}".format(a.balance.aergo))
            print("        + gaer               = {}".format(a.balance.gaer))
            print("        + aer                = {}".format(a.balance.aer))
            print("    - nonce                  = {}".format(a.nonce))
            print("    - code hash              = {}".format(a.code_hash))
            print("    - storage root           = {}".format(a.storage_root))
            print("    - sql recovery point     = {}".format(a.sql_recovery_point))

        print("------ Disconnect AERGO -----------")
        aergo.disconnect()
    except Exception as e:
        eprint(e)
        traceback.print_exception(*sys.exc_info())


if __name__ == '__main__':
    run()



# -*- coding: utf-8 -*-

import ecdsa
import enum

from ..utils.encoding import encode_address, decode_address
from ..utils.converter import convert_bytes_to_public_key, \
    convert_public_key_to_bytes


MAX_NAME_ADDRESS_LEN = 12


@enum.unique
class GovernanceTxAddress(enum.Enum):
    SYSTEM = "aergo.system"
    NAME = "aergo.name"
    ENTERPRISE = "aergo.enterprise"


def check_name_address(addr):
    if len(addr) <= MAX_NAME_ADDRESS_LEN:
        return 1
    elif addr in set(e.value for e in GovernanceTxAddress):
        return 2
    else:
        return 0


class Address:
    def __init__(self, pubkey, empty=False, curve=ecdsa.SECP256k1):
        self.__address = None
        self.__curve = curve
        self.__empty = empty

        if empty:
            return

        if pubkey is None:
            assert 1 == 0

        if isinstance(pubkey, str):
            pubkey = decode_address(pubkey)
        elif isinstance(pubkey, bytes):
            pubkey = convert_bytes_to_public_key(pubkey, curve=curve)

        self.__address = convert_public_key_to_bytes(pubkey=pubkey,
                                                     curve=curve,
                                                     compressed=True)

    def __str__(self):
        return self.encode(self.__address)

    def __bytes__(self):
        return self.__address

    @property
    def value(self):
        return self.__address

    @value.setter
    def value(self, v):
        if self.__empty:
            if isinstance(v, str):
                self.__address = self.decode(v)
            elif isinstance(v, bytes):
                self.__address = v
        else:
            raise ValueError('Cannot set a value for the derived address')

    @property
    def curve(self):
        return self.__curve

    @property
    def public_key(self):
        return convert_bytes_to_public_key(self.__address, curve=self.__curve)

    @staticmethod
    def encode(addr):
        try:
            if addr is None or 0 == len(addr):
                return ''
            elif len(addr) < 32:
                return str(addr, 'UTF-8')
        except:
            pass

        return encode_address(addr)

    @staticmethod
    def decode(addr):
        try:
            if addr is None or 0 == len(addr):
                return b''
            elif check_name_address(addr) > 0:
                return addr.encode()
        except:
            pass

        return decode_address(addr)



# -*- coding: utf-8 -*-

import base58
import base64
import ecdsa

from ..constants import *


def is_empty(v):
    if v is None or 0 == len(v):
        return True
    return False


def encode_b64(v):
    if is_empty(v):
        return None
    return base64.b64encode(v).decode('utf-8')


def decode_b64(v):
    if is_empty(v):
        return None
    return base64.b64decode(v)


def encode_b58_check(v):
    if is_empty(v):
        return None
    return base58.b58encode_check(v).decode('utf-8')


def decode_b58_check(v):
    if is_empty(v):
        return None
    return base58.b58decode_check(v)


def encode_b58(v):
    if is_empty(v):
        return None
    return base58.b58encode(v).decode('utf-8')


def decode_b58(v):
    if is_empty(v):
        return None
    return base58.b58decode(v)


def encode_address(address):
    v = ADDRESS_VERSION + address
    return encode_b58_check(v)


def decode_address(address):
    if is_empty(address):
        return None
    v = decode_b58_check(address)
    return v[len(ADDRESS_VERSION):]

def decode_root(root):
    if is_empty(root):
        return None
    return base58.b58decode(root)


def encode_payload(payload):
    if is_empty(payload):
        return None
    return encode_b58_check(payload)


def decode_payload(payload_str):
    if is_empty(payload_str):
        return None
    return decode_b58_check(payload_str)


def encode_private_key(private_key):
    v = PRIVATE_KEY_VERSION + private_key
    return encode_b58_check(v)


def decode_private_key(private_key):
    if is_empty(private_key):
        return None
    v = decode_b58_check(private_key)
    return v[len(PRIVATE_KEY_VERSION):]


def encode_tx_hash(tx_hash):
    if is_empty(tx_hash):
        return None
    return encode_b58(tx_hash)


def decode_tx_hash(tx_hash):
    if is_empty(tx_hash):
        return None
    return decode_b58(tx_hash)


def encode_signature(sign):
    if is_empty(sign):
        return None
    return encode_b58(sign)


def decode_signature(sign):
    if is_empty(sign):
        return None
    return decode_b58(sign)


def decode_public_key(public_key, curve=ecdsa.SECP256k1):
    v = decode_address(public_key)
    head = v[:1]
    x_bytes = v[1:curve.baselen]

    if PUBLIC_KEY_UNCOMPRESSED == head:
        y_bytes = v[curve.baselen+1:]
    else:
        y_bytes = None
    return head, x_bytes, y_bytes


def encode_block_hash(block_hash):
    if is_empty(block_hash):
        return None
    return encode_b58(block_hash)


def decode_block_hash(block_hash):
    if is_empty(block_hash):
        return None
    return decode_b58(block_hash)



# -*- coding: utf-8 -*-

import grpc
import grpc._channel


class Stream:
    def __init__(self, grpc_stream):
        self._grpc_stream = grpc_stream

        self.__started = False
        self.__stopped = False

    def start(self):
        self.__started = True
        self.__stopped = False

    def stop(self):
        self.__started = False
        self.__stopped = True

    @property
    def started(self):
        return self.__started

    @property
    def stopped(self):
        return self.__stopped

    def __iter__(self):
        return self

    def __next__(self):
        try:
            return self.next()
        except grpc._channel._Rendezvous:
            raise RuntimeError("stream is cancelled")

    def next(self):
        # need to implement for each subclass
        pass

    def cancel(self):
        self._grpc_stream.cancel()

    def cancelled(self):
        return self._grpc_stream.cancelled()

    def done(self):
        return self._grpc_stream.done()

    def is_active(self):
        return self._grpc_stream.is_active()

    def running(self):
        return self._grpc_stream.running()

    """
    def result(self, timeout):
        try:
            result = self._grpc_stream.result(timeout)
        except grpc.FutureTimeoutError:
            raise TimeoutError
        return result

    def add_callback(self, fn):
        result = self._grpc_stream.add_callback(fn)
        return result
        
    def exception(self, timeout):
        try:
            result = self._grpc_stream.exception(timeout)
        except grpc.FutureTimeoutError:
            raise TimeoutError
        return result
        
    def time_remaining(self):
        result = self._grpc_stream.time_remaining()
        return result
    """



# -*- coding: utf-8 -*-

import json

from .block_hash import BlockHash
from .peer import Peer
from ..grpc.rpc_pb2 import ConsensusInfo as CInfo


def _get_dict_value(d, k):
    if d is None:
        return None

    if k in d:
        return d[k]
    else:
        return None


class ConsensusInfo:
    def __init__(self, info, consensus_type=None):
        if isinstance(info, str):
            info = json.loads(info)
            self._info = info
            self._info_detail = None
            self._status = None
            self._type = _get_dict_value(info, 'Type')
            self._status = _get_dict_value(info, 'Status')
            lib_hash = _get_dict_value(info, 'LibHash')
            if lib_hash is not None:
                self._lib_hash = BlockHash(lib_hash)
            else:
                self._lib_hash = None
            self._lib_no = _get_dict_value(info, 'LibNo')
            self._block_producer_list = []
        elif type(info) == CInfo:
            self._info = info
            self._type = info.type
            if self._type == 'raft':
                self._info_detail = json.loads(info.info)
                self._status = _get_dict_value(self._info_detail, 'Status')
                self._lib_hash = None
                self._lib_no = None
                self._block_producer_list = []
                for bp in info.bps:
                    self._block_producer_list.append(json.loads(bp))
            else:
                self._info_detail = None
                self._status = None
                self._lib_hash = None
                self._lib_no = None
                self._block_producer_list = []
                for bp in info.bps:
                    self._block_producer_list.append(json.loads(bp))
        elif consensus_type:
            self._info = None
            self._info_detail = None
            self._status = None
            self._type = consensus_type
            self._status = None
            self._lib_hash = None
            self._lib_no = None
            self._block_producer_list = []
        else:
            raise ValueError("Cannot recognize the value: {}".format(info))

    @property
    def type(self):
        return self._type

    @property
    def detail(self):
        return self._info_detail

    @property
    def status(self):
        return self._status

    @property
    def lib_hash(self):
        """
        get the last irreversible block (LIB) hash
        :return:
        """
        return self._lib_hash

    @property
    def lib_no(self):
        """
        get the last irreversible block (LIB) number
        :return:
        """
        return self._lib_no

    @property
    def block_producer_list(self):
        return self._block_producer_list

    def json(self):
        return {
            'type': self.type,
            'status': self.status,
            'lib_hash': self.lib_hash,
            'lib_no': self.lib_no,
            'block_producer_list': self.block_producer_list,
            'detail': self.detail,
        }

    def __str__(self):
        return json.dumps(self.json(), indent=2)




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: raft.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import p2p_pb2 as p2p__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='raft.proto',
  package='types',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\nraft.proto\x12\x05types\x1a\tp2p.proto\"G\n\nMemberAttr\x12\n\n\x02ID\x18\x01 \x01(\x04\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07\x61\x64\x64ress\x18\x03 \x01(\t\x12\x0e\n\x06peerID\x18\x04 \x01(\x0c\"q\n\x10MembershipChange\x12)\n\x04type\x18\x01 \x01(\x0e\x32\x1b.types.MembershipChangeType\x12\x11\n\trequestID\x18\x02 \x01(\x04\x12\x1f\n\x04\x61ttr\x18\x03 \x01(\x0b\x32\x11.types.MemberAttr\"8\n\x15MembershipChangeReply\x12\x1f\n\x04\x61ttr\x18\x01 \x01(\x0b\x32\x11.types.MemberAttr\"-\n\rHardStateInfo\x12\x0c\n\x04term\x18\x01 \x01(\x04\x12\x0e\n\x06\x63ommit\x18\x02 \x01(\x04\".\n\x15GetClusterInfoRequest\x12\x15\n\rbestBlockHash\x18\x01 \x01(\x0c\"\xb2\x01\n\x16GetClusterInfoResponse\x12\x0f\n\x07\x63hainID\x18\x01 \x01(\x0c\x12\x11\n\tclusterID\x18\x02 \x01(\x04\x12\r\n\x05\x65rror\x18\x03 \x01(\t\x12#\n\x08mbrAttrs\x18\x04 \x03(\x0b\x32\x11.types.MemberAttr\x12\x13\n\x0b\x62\x65stBlockNo\x18\x05 \x01(\x04\x12+\n\rhardStateInfo\x18\x06 \x01(\x0b\x32\x14.types.HardStateInfo\"l\n\x12\x43onfChangeProgress\x12%\n\x05State\x18\x01 \x01(\x0e\x32\x16.types.ConfChangeState\x12\x0b\n\x03\x45rr\x18\x02 \x01(\t\x12\"\n\x07Members\x18\x03 \x03(\x0b\x32\x11.types.MemberAttr\"H\n\x10SnapshotResponse\x12#\n\x06status\x18\x01 \x01(\x0e\x32\x13.types.ResultStatus\x12\x0f\n\x07message\x18\x02 \x01(\t*9\n\x14MembershipChangeType\x12\x0e\n\nADD_MEMBER\x10\x00\x12\x11\n\rREMOVE_MEMBER\x10\x01*m\n\x0f\x43onfChangeState\x12\x1e\n\x1a\x43ONF_CHANGE_STATE_PROPOSED\x10\x00\x12\x1b\n\x17\x43ONF_CHANGE_STATE_SAVED\x10\x01\x12\x1d\n\x19\x43ONF_CHANGE_STATE_APPLIED\x10\x02\x62\x06proto3')
  ,
  dependencies=[p2p__pb2.DESCRIPTOR,])

_MEMBERSHIPCHANGETYPE = _descriptor.EnumDescriptor(
  name='MembershipChangeType',
  full_name='types.MembershipChangeType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ADD_MEMBER', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REMOVE_MEMBER', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=738,
  serialized_end=795,
)
_sym_db.RegisterEnumDescriptor(_MEMBERSHIPCHANGETYPE)

MembershipChangeType = enum_type_wrapper.EnumTypeWrapper(_MEMBERSHIPCHANGETYPE)
_CONFCHANGESTATE = _descriptor.EnumDescriptor(
  name='ConfChangeState',
  full_name='types.ConfChangeState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CONF_CHANGE_STATE_PROPOSED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CONF_CHANGE_STATE_SAVED', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CONF_CHANGE_STATE_APPLIED', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=797,
  serialized_end=906,
)
_sym_db.RegisterEnumDescriptor(_CONFCHANGESTATE)

ConfChangeState = enum_type_wrapper.EnumTypeWrapper(_CONFCHANGESTATE)
ADD_MEMBER = 0
REMOVE_MEMBER = 1
CONF_CHANGE_STATE_PROPOSED = 0
CONF_CHANGE_STATE_SAVED = 1
CONF_CHANGE_STATE_APPLIED = 2



_MEMBERATTR = _descriptor.Descriptor(
  name='MemberAttr',
  full_name='types.MemberAttr',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ID', full_name='types.MemberAttr.ID', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='types.MemberAttr.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='address', full_name='types.MemberAttr.address', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='peerID', full_name='types.MemberAttr.peerID', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=32,
  serialized_end=103,
)


_MEMBERSHIPCHANGE = _descriptor.Descriptor(
  name='MembershipChange',
  full_name='types.MembershipChange',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='types.MembershipChange.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='requestID', full_name='types.MembershipChange.requestID', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attr', full_name='types.MembershipChange.attr', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=105,
  serialized_end=218,
)


_MEMBERSHIPCHANGEREPLY = _descriptor.Descriptor(
  name='MembershipChangeReply',
  full_name='types.MembershipChangeReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attr', full_name='types.MembershipChangeReply.attr', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=220,
  serialized_end=276,
)


_HARDSTATEINFO = _descriptor.Descriptor(
  name='HardStateInfo',
  full_name='types.HardStateInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='term', full_name='types.HardStateInfo.term', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='commit', full_name='types.HardStateInfo.commit', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=278,
  serialized_end=323,
)


_GETCLUSTERINFOREQUEST = _descriptor.Descriptor(
  name='GetClusterInfoRequest',
  full_name='types.GetClusterInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bestBlockHash', full_name='types.GetClusterInfoRequest.bestBlockHash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=325,
  serialized_end=371,
)


_GETCLUSTERINFORESPONSE = _descriptor.Descriptor(
  name='GetClusterInfoResponse',
  full_name='types.GetClusterInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chainID', full_name='types.GetClusterInfoResponse.chainID', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='clusterID', full_name='types.GetClusterInfoResponse.clusterID', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error', full_name='types.GetClusterInfoResponse.error', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mbrAttrs', full_name='types.GetClusterInfoResponse.mbrAttrs', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bestBlockNo', full_name='types.GetClusterInfoResponse.bestBlockNo', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hardStateInfo', full_name='types.GetClusterInfoResponse.hardStateInfo', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=374,
  serialized_end=552,
)


_CONFCHANGEPROGRESS = _descriptor.Descriptor(
  name='ConfChangeProgress',
  full_name='types.ConfChangeProgress',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='State', full_name='types.ConfChangeProgress.State', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Err', full_name='types.ConfChangeProgress.Err', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Members', full_name='types.ConfChangeProgress.Members', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=554,
  serialized_end=662,
)


_SNAPSHOTRESPONSE = _descriptor.Descriptor(
  name='SnapshotResponse',
  full_name='types.SnapshotResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='types.SnapshotResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='types.SnapshotResponse.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=664,
  serialized_end=736,
)

_MEMBERSHIPCHANGE.fields_by_name['type'].enum_type = _MEMBERSHIPCHANGETYPE
_MEMBERSHIPCHANGE.fields_by_name['attr'].message_type = _MEMBERATTR
_MEMBERSHIPCHANGEREPLY.fields_by_name['attr'].message_type = _MEMBERATTR
_GETCLUSTERINFORESPONSE.fields_by_name['mbrAttrs'].message_type = _MEMBERATTR
_GETCLUSTERINFORESPONSE.fields_by_name['hardStateInfo'].message_type = _HARDSTATEINFO
_CONFCHANGEPROGRESS.fields_by_name['State'].enum_type = _CONFCHANGESTATE
_CONFCHANGEPROGRESS.fields_by_name['Members'].message_type = _MEMBERATTR
_SNAPSHOTRESPONSE.fields_by_name['status'].enum_type = p2p__pb2._RESULTSTATUS
DESCRIPTOR.message_types_by_name['MemberAttr'] = _MEMBERATTR
DESCRIPTOR.message_types_by_name['MembershipChange'] = _MEMBERSHIPCHANGE
DESCRIPTOR.message_types_by_name['MembershipChangeReply'] = _MEMBERSHIPCHANGEREPLY
DESCRIPTOR.message_types_by_name['HardStateInfo'] = _HARDSTATEINFO
DESCRIPTOR.message_types_by_name['GetClusterInfoRequest'] = _GETCLUSTERINFOREQUEST
DESCRIPTOR.message_types_by_name['GetClusterInfoResponse'] = _GETCLUSTERINFORESPONSE
DESCRIPTOR.message_types_by_name['ConfChangeProgress'] = _CONFCHANGEPROGRESS
DESCRIPTOR.message_types_by_name['SnapshotResponse'] = _SNAPSHOTRESPONSE
DESCRIPTOR.enum_types_by_name['MembershipChangeType'] = _MEMBERSHIPCHANGETYPE
DESCRIPTOR.enum_types_by_name['ConfChangeState'] = _CONFCHANGESTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MemberAttr = _reflection.GeneratedProtocolMessageType('MemberAttr', (_message.Message,), {
  'DESCRIPTOR' : _MEMBERATTR,
  '__module__' : 'raft_pb2'
  # @@protoc_insertion_point(class_scope:types.MemberAttr)
  })
_sym_db.RegisterMessage(MemberAttr)

MembershipChange = _reflection.GeneratedProtocolMessageType('MembershipChange', (_message.Message,), {
  'DESCRIPTOR' : _MEMBERSHIPCHANGE,
  '__module__' : 'raft_pb2'
  # @@protoc_insertion_point(class_scope:types.MembershipChange)
  })
_sym_db.RegisterMessage(MembershipChange)

MembershipChangeReply = _reflection.GeneratedProtocolMessageType('MembershipChangeReply', (_message.Message,), {
  'DESCRIPTOR' : _MEMBERSHIPCHANGEREPLY,
  '__module__' : 'raft_pb2'
  # @@protoc_insertion_point(class_scope:types.MembershipChangeReply)
  })
_sym_db.RegisterMessage(MembershipChangeReply)

HardStateInfo = _reflection.GeneratedProtocolMessageType('HardStateInfo', (_message.Message,), {
  'DESCRIPTOR' : _HARDSTATEINFO,
  '__module__' : 'raft_pb2'
  # @@protoc_insertion_point(class_scope:types.HardStateInfo)
  })
_sym_db.RegisterMessage(HardStateInfo)

GetClusterInfoRequest = _reflection.GeneratedProtocolMessageType('GetClusterInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETCLUSTERINFOREQUEST,
  '__module__' : 'raft_pb2'
  # @@protoc_insertion_point(class_scope:types.GetClusterInfoRequest)
  })
_sym_db.RegisterMessage(GetClusterInfoRequest)

GetClusterInfoResponse = _reflection.GeneratedProtocolMessageType('GetClusterInfoResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETCLUSTERINFORESPONSE,
  '__module__' : 'raft_pb2'
  # @@protoc_insertion_point(class_scope:types.GetClusterInfoResponse)
  })
_sym_db.RegisterMessage(GetClusterInfoResponse)

ConfChangeProgress = _reflection.GeneratedProtocolMessageType('ConfChangeProgress', (_message.Message,), {
  'DESCRIPTOR' : _CONFCHANGEPROGRESS,
  '__module__' : 'raft_pb2'
  # @@protoc_insertion_point(class_scope:types.ConfChangeProgress)
  })
_sym_db.RegisterMessage(ConfChangeProgress)

SnapshotResponse = _reflection.GeneratedProtocolMessageType('SnapshotResponse', (_message.Message,), {
  'DESCRIPTOR' : _SNAPSHOTRESPONSE,
  '__module__' : 'raft_pb2'
  # @@protoc_insertion_point(class_scope:types.SnapshotResponse)
  })
_sym_db.RegisterMessage(SnapshotResponse)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-

import enum
import json

from google.protobuf.json_format import MessageToJson
from ..grpc.raft_pb2 import ConfChangeState


@enum.unique
class ChangeConfState(enum.Enum):
    """ ChangeConfState holds the state of the request 'changeCluster'
    to change configuration of RAFT cluster.
    """
    PROPOSED = ConfChangeState.Value("CONF_CHANGE_STATE_PROPOSED")
    SAVED = ConfChangeState.Value("CONF_CHANGE_STATE_SAVED")
    APPLIED = ConfChangeState.Value("CONF_CHANGE_STATE_APPLIED")


class ChangeConfInfo:
    """ ChangeConfInfo shows the state of the request 'changeCluster'
    to change configuration of RAFT cluster and member list of the cluster.
    """
    def __init__(self, info):
        self.__info = json.loads(MessageToJson(info))
        self.__state = ChangeConfState(info.State)
        self.__members = self.__info["Members"]
        self.__err = info.Err

    def __str__(self):
        return json.dumps(self.json(), indent=2)

    def json(self):
        return self.__info

    @property
    def state(self):
        return self.__state

    @property
    def members(self):
        return self.__members

    @property
    def error(self):
        return self.__err



import pytest

import decimal

import aergo.herapy as herapy

from aergo.herapy.obj.aer import AERGO_UNIT_PRECISION


def test_success():
    aer = herapy.Aer('1aergo')
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 1000000000000000000 == aer.dec
    assert '1 aergo' == aer.aergo
    assert '1000000000 gaer' == aer.gaer
    assert '1000000000000000000 aer' == aer.aer

    aer = herapy.Aer('1 gaer')
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 1000000000 == aer.dec
    assert '0.000000001 aergo' == aer.aergo
    assert '1 gaer' == aer.gaer
    assert '1000000000 aer' == aer.aer

    aer = herapy.Aer('1')
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 1 == aer.dec
    assert '0.000000000000000001 aergo' == aer.aergo
    assert '0.000000001 gaer' == aer.gaer
    assert '1 aer' == aer.aer

    aer = herapy.Aer('1.01 aergo')
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 1010000000000000000 == aer.dec
    assert '1.01 aergo' == aer.aergo
    assert '1010000000 gaer' == aer.gaer
    assert '1010000000000000000 aer' == aer.aer

    aer = herapy.Aer('1.01gaer')
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 1010000000 == aer.dec
    assert '0.00000000101 aergo' == aer.aergo
    assert '1.01 gaer' == aer.gaer
    assert '1010000000 aer' == aer.aer

    aer = herapy.Aer('1 aer')
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 1 == aer.dec
    assert '0.000000000000000001 aergo' == aer.aergo
    assert '0.000000001 gaer' == aer.gaer
    assert '1 aer' == aer.aer

    aer = herapy.Aer('0.123456789012345678 aergo')
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 123456789012345678 == aer.dec
    assert '0.123456789012345678 aergo' == aer.aergo
    assert '123456789.012345678 gaer' == aer.gaer
    assert '123456789012345678 aer' == aer.aer

    # max length of decimal
    aer = herapy.Aer('0.100000000000000001 aergo')
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 100000000000000001 == aer.dec
    assert '0.100000000000000001 aergo' == aer.aergo
    assert '100000000.000000001 gaer' == aer.gaer
    assert '100000000000000001 aer' == aer.aer

    # max length of decimal
    aer = herapy.Aer('499999999.100000000000000001 aergo')
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 499999999100000000000000001 == aer.dec
    assert '499999999.100000000000000001 aergo' == aer.aergo
    assert '499999999100000000.000000001 gaer' == aer.gaer
    assert '499999999100000000000000001 aer' == aer.aer

    # max length of decimal
    aer = herapy.Aer('499999999100000000000000001 aer')
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 499999999100000000000000001 == aer.dec
    assert '499999999.100000000000000001 aergo' == aer.aergo
    assert '499999999100000000.000000001 gaer' == aer.gaer
    assert '499999999100000000000000001 aer' == aer.aer

    aer = herapy.Aer('24999999910000000081000000100000000000000000000000000000000000.0 aergo')
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert decimal.Decimal('24999999910000000081000000100000000000000000000000000000000000000000000000000000') == aer.dec
    assert '24999999910000000081000000100000000000000000000000000000000000 aergo' == aer.aergo
    assert '24999999910000000081000000100000000000000000000000000000000000000000000 gaer' == aer.gaer
    assert '24999999910000000081000000100000000000000000000000000000000000000000000000000000 aer' == aer.aer

    aer1 = herapy.Aer('24999999910000000081000000100000000000000000000000000000000000000000000000000000 aer')
    aer2 = herapy.Aer('24999999910000000081000000100000000000000000000000000000000000 aergo')
    assert aer1.dec == aer2.dec
    assert str(aer1) == str(aer2)
    assert int(aer1) == int(aer2)
    assert bytes(aer1) == bytes(aer2)

    aer3 = herapy.Aer()
    aer3.dec = 24999999910000000081000000100000000000000000000000000000000000000000000000000000
    assert aer1.dec == aer3.dec
    assert str(aer1) == str(aer3)
    assert int(aer1) == int(aer3)
    assert bytes(aer1) == bytes(aer3)

    aer = herapy.Aer(b'test')
    assert 1952805748 == aer.dec
    assert '1952805748 aer' == aer.aer
    assert '1.952805748 gaer' == aer.gaer
    assert '0.000000001952805748 aergo' == aer.aergo

    with pytest.raises(herapy.errors.ConversionException):
        herapy.Aer(0.1)

    aer = herapy.Aer()
    aer.dec = 1
    assert 1 == aer.dec
    assert '1 aer' == aer.aer
    assert '0.000000001 gaer' == aer.gaer
    assert '0.000000000000000001 aergo' == aer.aergo

    with pytest.raises(herapy.errors.ConversionException):
        aer = herapy.Aer()
        aer.dec = 0.1

    with pytest.raises(herapy.errors.ConversionException):
        aer = herapy.Aer()
        aer.dec = 0.0001 / AERGO_UNIT_PRECISION

    aer = herapy.Aer(10000000000000000 / 1)
    assert '1000000000000000 aer' == aer.aer
    assert '1000000 gaer' == aer.gaer
    assert '0.001 aergo' == aer.aergo

    aer = herapy.Aer()
    aer.dec = 10000000000000000 / 1
    assert '1000000000000000 aer' == aer.aer
    assert '1000000 gaer' == aer.gaer
    assert '0.001 aergo' == aer.aergo

    with pytest.raises(herapy.errors.ConversionException):
        aer = herapy.Aer()
        aer.dec = 10000000000000000 / 3

    aer = herapy.Aer(123141000000000000000000 / 10)
    assert '12314100000000000000000 aer' == aer.aer
    assert '12314100000000 gaer' == aer.gaer
    assert '12314.1 aergo' == aer.aergo


def test_fail():
    with pytest.raises(herapy.errors.ConversionException):
        herapy.Aer('1 aergogo')
    with pytest.raises(herapy.errors.ConversionException):
        herapy.Aer('1gaergogo')
    with pytest.raises(herapy.errors.ConversionException):
        herapy.Aer('1.01')
    with pytest.raises(herapy.errors.ConversionException):
        herapy.Aer('1.01.0')
    with pytest.raises(herapy.errors.ConversionException):
        herapy.Aer('1.01 aer')
    with pytest.raises(herapy.errors.ConversionException):
        herapy.Aer('0.0000000000000000001 aergo')
    with pytest.raises(herapy.errors.ConversionException):
        herapy.Aer('499999999100000000000000001.1 aer')
    with pytest.raises(herapy.errors.ConversionException):
        herapy.Aer('abc.abc aergo')
    with pytest.raises(herapy.errors.ConversionException):
        herapy.Aer('')


def test_op_add():
    aer1 = herapy.Aer('1')
    aer2 = herapy.Aer('10')
    aer = aer1 + aer2
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 11 == aer.dec
    assert '0.000000000000000011 aergo' == aer.aergo
    assert '0.000000011 gaer' == aer.gaer
    assert '11 aer' == aer.aer

    aer1 = herapy.Aer('499999999100000000000000001 aer')
    aer2 = herapy.Aer('499999999100000000000000001 aer')
    assert str(aer1) == str(aer2)
    assert int(aer1) == int(aer2)
    assert bytes(aer1) == bytes(aer2)

    aer = aer1 + aer2
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 999999998200000000000000002 == aer.dec
    assert '999999998.200000000000000002 aergo' == aer.aergo
    assert '999999998200000000.000000002 gaer' == aer.gaer
    assert '999999998200000000000000002 aer' == aer.aer


def test_op_sub():
    aer1 = herapy.Aer('1')
    aer2 = herapy.Aer('10')
    aer = aer1 - aer2
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert -9 == aer.dec
    assert '-0.000000000000000009 aergo' == aer.aergo
    assert '-0.000000009 gaer' == aer.gaer
    assert '-9 aer' == aer.aer

    aer1 = herapy.Aer('499999999100000000000000001 aer')
    aer2 = herapy.Aer('499999999100000000000000001 aer')
    assert str(aer1) == str(aer2)
    assert int(aer1) == int(aer2)
    assert bytes(aer1) == bytes(aer2)

    aer = aer1 - aer2
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 0 == aer.dec
    assert '0 aergo' == aer.aergo
    assert '0 gaer' == aer.gaer
    assert '0 aer' == aer.aer


def test_op_mul():
    aer1 = herapy.Aer('1')
    aer2 = herapy.Aer('10')
    aer = aer1 * aer2
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 10 == aer.dec
    assert '0.00000000000000001 aergo' == aer.aergo
    assert '0.00000001 gaer' == aer.gaer
    assert '10 aer' == aer.aer

    aer1 = herapy.Aer('499999999100000000000000001 aer')
    aer2 = herapy.Aer('499999999100000000000000001 aer')
    assert str(aer1) == str(aer2)
    assert int(aer1) == int(aer2)
    assert bytes(aer1) == bytes(aer2)

    aer = aer1 * aer2
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 249999999100000000810000000999999998200000000000000001 == aer.dec
    assert '249999999100000000810000000999999998.200000000000000001 aergo' == aer.aergo
    assert '249999999100000000810000000999999998200000000.000000001 gaer' == aer.gaer
    assert '249999999100000000810000000999999998200000000000000001 aer' == aer.aer


def test_op_div():
    aer1 = herapy.Aer('2')
    aer2 = herapy.Aer('10')

    with pytest.raises(herapy.errors.ConversionException):
        aer1 / aer2

    aer = aer2 / aer1
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 5 == aer.dec
    assert '0.000000000000000005 aergo' == aer.aergo
    assert '0.000000005 gaer' == aer.gaer
    assert '5 aer' == aer.aer

    aer1 = herapy.Aer('499999999100000000000000001 aer')
    aer2 = herapy.Aer('499999999100000000000000001 aer')
    assert str(aer1) == str(aer2)
    assert int(aer1) == int(aer2)
    assert bytes(aer1) == bytes(aer2)

    aer = aer1 // aer2
    assert isinstance(aer, herapy.Aer)
    assert aer.aer == str(aer)
    assert 1 == aer.dec
    assert '0.000000000000000001 aergo' == aer.aergo
    assert '0.000000001 gaer' == aer.gaer
    assert '1 aer' == aer.aer



import sys
import traceback

import aergo.herapy as herapy


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)
    print(*args, **kwargs)


def run():
    try:
        SC_ADDRESS = herapy.GovernanceTxAddress.ENTERPRISE
        """
        const SetConf = "setConf"
        const AppendConf = "appendConf"
        const RemoveConf = "removeConf"
        const AppendAdmin = "appendAdmin"
        const RemoveAdmin = "removeAdmin"
        const EnableConf = "enableConf"
        const DisableConf = "disableConf"
        const ChangeCluster = "changeCluster"
        """
        aergo = herapy.Aergo()

        print("------ Connect AERGO -----------")
        aergo.connect('localhost:7845')

        print("------ Set Admin Account -----------")
        sender_private_key = "6hbRWgddqcg2ZHE5NipM1xgwBDAKqLnCKhGvADWrWE18xAbX8sW"
        sender_account = aergo.new_account(private_key=sender_private_key)
        print("  > Admin Address: {}".format(sender_account.address))
        print(herapy.utils.convert_bytes_to_int_str(bytes(sender_account.address)))

        aergo.get_account()
        print("    > account state of Sender")
        print("      - balance        = {}".format(sender_account.balance))
        print("      - nonce          = {}".format(sender_account.nonce))
        print("      - code hash      = {}".format(sender_account.code_hash))
        print("      - storage root   = {}".format(sender_account.storage_root))

        print("------ Call SC: appendAdmin -----------")
        tx, result = aergo.call_sc(SC_ADDRESS, "appendAdmin", args=[str(sender_account.address)])

        print("-------Wait for tx result--------")
        result = aergo.wait_tx_result(tx.tx_hash)
        if result.status != herapy.TxResultStatus.SUCCESS:
            eprint("  > ERROR[{0}]:{1}: {2}".format(
                result.contract_address, result.status, result.detail))
            aergo.disconnect()
            return

        print("------ Check result of Call SC -----------")
        print("  > TX: {}".format(tx.tx_hash))
        result = aergo.get_tx_result(tx.tx_hash)
        if result.status != herapy.TxResultStatus.SUCCESS:
            eprint("  > ERROR[{0}]:{1}: {2}".format(
                result.contract_address, result.status, result.detail))
            aergo.disconnect()
            return

        config = aergo.get_enterprise_config('ADMINS')
        print(config.values)

        print("------ Call SC: changeCluster -----------")
        args = {
            "command": "add",
            "name": "aergonew",
            "address": "/ip4/127.0.0.1/tcp/11001",
            "peerid": "16Uiu2HAmAAtqye6QQbeG9EZnrWJbGK8Xw74cZxpnGGEAZAB3zJ8B"
        }
        tx, result = aergo.call_sc(SC_ADDRESS, "changeCluster",
                                   args=[args])

        print("-------Wait for tx result--------")
        result = aergo.wait_tx_result(tx.tx_hash)
        if result.status != herapy.TxResultStatus.SUCCESS:
            eprint("  > ERROR[{0}]:{1}: {2}".format(
                result.contract_address, result.status, result.detail))
            aergo.disconnect()
            return

        print("------ Check result of Call SC -----------")
        print("  > TX: {}".format(tx.tx_hash))
        result = aergo.get_tx_result(tx.tx_hash)
        if result.status != herapy.TxResultStatus.SUCCESS:
            eprint("  > ERROR[{0}]:{1}: {2}".format(
                result.contract_address, result.status, result.detail))
            aergo.disconnect()
            return

        status = aergo.get_conf_change_progress(result.block_no)
        print(status)

        print("------ Call SC: removeAdmin -----------")
        tx, result = aergo.call_sc(SC_ADDRESS, "removeAdmin", args=[str(sender_account.address)])

        print("-------Wait for tx result--------")
        result = aergo.wait_tx_result(tx.tx_hash)
        print(result)
        if result.status != herapy.TxResultStatus.SUCCESS:
            eprint("  > ERROR[{0}]:{1}: {2}".format(
                result.contract_address, result.status, result.detail))
            aergo.disconnect()
            return

        print("------ Check result of Call SC -----------")
        print("  > TX: {}".format(tx.tx_hash))
        result = aergo.get_tx_result(tx.tx_hash)
        if result.status != herapy.TxResultStatus.SUCCESS:
            eprint("  > ERROR[{0}]:{1}: {2}".format(
                result.contract_address, result.status, result.detail))
            aergo.disconnect()
            return

        print("------ Disconnect AERGO -----------")
        aergo.disconnect()
    except Exception as e:
        eprint(e)
        traceback.print_exception(*sys.exc_info())


if __name__ == '__main__':
    run()
